#  tests for cvxopt-1.3.0-py310h7aab8bc_0 (this is a generated file);
print('===== testing package: cvxopt-1.3.0-py310h7aab8bc_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
#!/usr/bin/env python
# -*- coding: utf-8 -*-

import cvxopt
import cvxopt.base
import cvxopt.blas
import cvxopt.cholmod
import cvxopt.lapack
import cvxopt.misc_solvers
import cvxopt.umfpack
import cvxopt.glpk
import cvxopt.fftw

import sys
import pytest

if __name__ == '__main__':
    sys.exit(pytest.main())
#  --- run_test.py (end) ---

print('===== cvxopt-1.3.0-py310h7aab8bc_0 OK =====');
